package it.danieleverducci.subitobeers.ui

import android.os.Bundle
import android.view.*
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import android.widget.Toast
import it.danieleverducci.subitobeers.BeerNavigation
import it.danieleverducci.subitobeers.BeersRepository
import it.danieleverducci.subitobeers.R
import it.danieleverducci.subitobeers.databinding.FragmentBeersListBinding
import it.danieleverducci.subitobeers.entities.Beer

/**
 * A fragment representing a list of Items.
 */
class BeersFragment : Fragment(), BeersRepository.Listener, BeerRecyclerAdapter.Listener {

    private val rvAdapter = BeerRecyclerAdapter()
    private val repo = BeersRepository()
    lateinit var binding: FragmentBeersListBinding
    private var page = 1

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        setHasOptionsMenu(true);

        binding = FragmentBeersListBinding.inflate(
            LayoutInflater.from(container!!.context),
            container,
            false
        )

        // Set the adapter
        with(binding.list) {
            layoutManager = LinearLayoutManager(context)
            adapter = rvAdapter
        }

        // Register for recyclerview adapter events
        rvAdapter.listener = this

        return binding.root
    }

    override fun onStart() {
        super.onStart()

        // Load beers from network
        repo.listener = this

        // TODO: evitare che ogni volta ricarichi gli stessi elementi
        page = 1
        repo.getBeers(page)
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        super.onCreateOptionsMenu(menu, inflater)
        requireActivity().menuInflater.inflate(R.menu.list_menu, menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (item.itemId == R.id.action_filter)
            toggleFilter()
        return super.onOptionsItemSelected(item)
    }

    override fun onBeersObtained(beers: List<Beer>) {
        rvAdapter.addItems(beers)
    }

    override fun onFailure() {
        Toast.makeText(context, R.string.network_error, Toast.LENGTH_SHORT).show()
    }

    /**
     * Called when an item is clicked in the list
     */
    override fun OnItemClicked(item: Beer) {
        if (activity is BeerNavigation)
            (activity as BeerNavigation).showBeerDetail(item);
        else
            throw IllegalStateException("Activity must implement BeerNavigation")
    }

    /**
     * Called when last list item is displayed.
     * Used to fetch more elements
     */
    override fun OnLastItemScrolled() {
        page++;
        repo.getBeers(page)
    }

    private fun toggleFilter() {
        binding.listFilterDropdown.visibility =
            if (binding.listFilterDropdown.visibility == View.VISIBLE) View.GONE else View.VISIBLE
    }

}